export declare type Render<E = unknown, O = void> = (element: E, options: O) => void | Promise<void>;
export declare type SuperRender<O = unknown, E = void> = ((render: (options: O) => void, element: E) => void);
export declare type AsyncSuperRender<O = unknown, E = void> = ((render: (options: O) => Promise<void>, element: E) => Promise<void>);
export declare type Customize<E = unknown, O = void> = (element: E, options: O) => void;
export declare type AsyncCustomize<E = unknown, O = void> = (element: E, options: O) => Promise<void>;
export declare const RefreshParent: void;
export interface Rtti<E = unknown, O = void> {
    readonly name: string;
    readonly sequential: boolean;
    initialize?(node: NodeInfo<E, O>): void;
    mount?(node: NodeInfo<E, O>): void;
    render?(node: NodeInfo<E, O>): void;
    finalize?(node: NodeInfo<E, O>, cause: NodeInfo): void;
}
export interface AbstractNodeInstance<E = unknown, O = void> {
    readonly uuid: number;
    readonly level: number;
    revision: number;
    native?: E;
    model?: unknown;
    children: ReadonlyArray<NodeInfo<any, any>>;
    buffer: Array<NodeInfo<any, any>> | undefined;
    aliens: ReadonlyArray<NodeInfo<any, any>>;
    resizing?: ResizeObserver;
    render(node: NodeInfo<E, O>): void;
}
export declare class NodeInfo<E = unknown, O = void> {
    readonly id: string;
    readonly args: unknown;
    readonly render: Render<E, O>;
    readonly superRender: SuperRender<O, E> | undefined;
    readonly priority: number;
    readonly rtti: Rtti<E, O>;
    readonly owner: NodeInfo;
    readonly host: NodeInfo;
    instance?: AbstractNodeInstance<E, O> | undefined;
    old?: NodeInfo<E, O>;
    sibling?: NodeInfo;
    get native(): E | undefined;
    constructor(id: string, args: unknown, render: Render<E, O>, superRender: SuperRender<O, E> | undefined, priority: number, rtti: Rtti<E, O>, owner: NodeInfo, host: NodeInfo, instance?: AbstractNodeInstance<E, O> | undefined);
}
