"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SvgRtti = exports.HtmlRtti = exports.AbstractHtmlRtti = void 0;
const reactronic_1 = require("reactronic");
const api_1 = require("../core/api");
class AbstractHtmlRtti {
    constructor(name, sequential = true) {
        this.name = name;
        this.sequential = sequential;
    }
    initialize(node) {
        const native = this.createElement(node);
        native.id = node.id;
        node.instance.native = native;
    }
    mount(node) {
        const self = node.instance;
        const native = self === null || self === void 0 ? void 0 : self.native;
        if (native) {
            const nativeHost = node.host.native;
            if (nativeHost instanceof Element) {
                const sibling = node.sibling;
                if (sibling === undefined) {
                    if (nativeHost !== native.parentNode || native.previousSibling !== null)
                        nativeHost.prepend(native);
                }
                else if (sibling !== node) {
                    if (sibling.host.native === nativeHost) {
                        const nativeSibling = sibling.native;
                        if (nativeSibling instanceof Element) {
                            if (nativeSibling.nextSibling !== native)
                                nativeHost.insertBefore(native, nativeSibling.nextSibling);
                        }
                    }
                }
                else
                    nativeHost.appendChild(native);
            }
        }
    }
    render(node) {
        const self = node.instance;
        const native = self.native;
        const outer = AbstractHtmlRtti.gNativeParent;
        try {
            AbstractHtmlRtti.gNativeParent = node;
            api_1.RxDom.render(node);
            AbstractHtmlRtti.blinkingEffect && blink(native, self.revision);
            if (AbstractHtmlRtti.isDebugAttributeEnabled)
                native.setAttribute('rdbg', `${self.revision}:    ${reactronic_1.Rx.why()}`);
        }
        finally {
            AbstractHtmlRtti.gNativeParent = outer;
        }
    }
    finalize(node, cause) {
        var _a, _b;
        const self = node.instance;
        const native = self === null || self === void 0 ? void 0 : self.native;
        if (!AbstractHtmlRtti.gSubTreeFinalization && native && native.parentElement) {
            AbstractHtmlRtti.gSubTreeFinalization = native;
            try {
                (_a = self.resizing) === null || _a === void 0 ? void 0 : _a.unobserve(native);
                native.remove();
                api_1.RxDom.finalize(node, cause);
            }
            finally {
                AbstractHtmlRtti.gSubTreeFinalization = undefined;
            }
        }
        else {
            if (native)
                (_b = self.resizing) === null || _b === void 0 ? void 0 : _b.unobserve(native);
            api_1.RxDom.finalize(node, cause);
        }
    }
    static get blinkingEffect() {
        return AbstractHtmlRtti._blinkingEffect;
    }
    static set blinkingEffect(value) {
        if (value === undefined) {
            api_1.RxDom.forAll((e) => {
                if (e instanceof HTMLElement)
                    e.classList.remove(`${AbstractHtmlRtti.blinkingEffect}-0`, `${AbstractHtmlRtti.blinkingEffect}-1`);
            });
        }
        AbstractHtmlRtti._blinkingEffect = value;
    }
}
exports.AbstractHtmlRtti = AbstractHtmlRtti;
AbstractHtmlRtti.isDebugAttributeEnabled = false;
AbstractHtmlRtti.gNativeParent = api_1.RxDom.createRootNode('html > body', true, global.document.body);
AbstractHtmlRtti.gSubTreeFinalization = undefined;
AbstractHtmlRtti._blinkingEffect = undefined;
function blink(e, cycle) {
    const n1 = cycle % 2;
    const n2 = 1 >> n1;
    e.classList.toggle(`${AbstractHtmlRtti.blinkingEffect}-${n1}`, true);
    e.classList.toggle(`${AbstractHtmlRtti.blinkingEffect}-${n2}`, false);
}
class HtmlRtti extends AbstractHtmlRtti {
    createElement(node) {
        return document.createElement(node.rtti.name);
    }
}
exports.HtmlRtti = HtmlRtti;
class SvgRtti extends AbstractHtmlRtti {
    createElement(node) {
        return document.createElementNS('http://www.w3.org/2000/svg', node.rtti.name);
    }
}
exports.SvgRtti = SvgRtti;
