"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ButtonSensor = exports.ButtonState = void 0;
const reactronic_1 = require("reactronic");
const BasePointerSensor_1 = require("./BasePointerSensor");
const DataForSensor_1 = require("./DataForSensor");
const KeyboardSensor_1 = require("./KeyboardSensor");
var ButtonState;
(function (ButtonState) {
    ButtonState[ButtonState["Pressed"] = 0] = "Pressed";
    ButtonState[ButtonState["Selecting"] = 1] = "Selecting";
    ButtonState[ButtonState["Selected"] = 2] = "Selected";
    ButtonState[ButtonState["Released"] = 3] = "Released";
})(ButtonState = exports.ButtonState || (exports.ButtonState = {}));
class ButtonSensor extends BasePointerSensor_1.BasePointerSensor {
    constructor(windowSensor) {
        super(windowSensor);
        this.state = ButtonState.Released;
        this.pointerButton = BasePointerSensor_1.PointerButton.None;
        this.originData = undefined;
        this.selectedData = undefined;
        this.selectedX = Infinity;
        this.selectedY = Infinity;
        this.selected = false;
    }
    listen(element, enabled = true) {
        const existing = this.sourceElement;
        if (element !== existing) {
            if (existing) {
                existing.removeEventListener('pointerdown', this.onPointerDown.bind(this), { capture: true });
                existing.removeEventListener('pointermove', this.onPointerMove.bind(this), { capture: true });
                existing.removeEventListener('pointerup', this.onPointerUp.bind(this), { capture: true });
                existing.removeEventListener('lostpointercapture', this.onLostPointerCapture.bind(this), { capture: true });
                existing.removeEventListener('keydown', this.onKeyDown.bind(this), { capture: true });
            }
            this.sourceElement = element;
            if (element && enabled) {
                element.addEventListener('pointerdown', this.onPointerDown.bind(this), { capture: true });
                element.addEventListener('pointermove', this.onPointerMove.bind(this), { capture: true });
                element.addEventListener('pointerup', this.onPointerUp.bind(this), { capture: true });
                element.addEventListener('lostpointercapture', this.onLostPointerCapture.bind(this), { capture: true });
                element.addEventListener('keydown', this.onKeyDown.bind(this), { capture: true });
            }
        }
    }
    onPointerDown(e) {
        if (this.state === ButtonState.Released && (this.pointerButton === BasePointerSensor_1.PointerButton.None))
            this.press(e);
        this.setPreventDefaultAndStopPropagation(e);
    }
    onPointerMove(e) {
        const state = this.state;
        if ((0, BasePointerSensor_1.isPointerButtonDown)(this.pointerButton, e.buttons)) {
            if (state === ButtonState.Pressed || state === ButtonState.Selecting)
                this.selecting(e);
        }
        else if (state !== ButtonState.Released) {
            this.cancel();
            this.reset();
        }
        this.setPreventDefaultAndStopPropagation(e);
    }
    onPointerUp(e) {
        const button = (0, BasePointerSensor_1.extractPointerButton)(e);
        if (button === this.pointerButton) {
            if (this.state === ButtonState.Selecting) {
                this.select(e);
                this.release();
            }
            else if (this.state === ButtonState.Pressed) {
                this.release();
            }
        }
        else
            this.cancel();
        this.reset();
        this.setPreventDefaultAndStopPropagation(e);
    }
    onLostPointerCapture(e) {
        if (this.state !== ButtonState.Released) {
            this.cancel();
            this.reset();
        }
    }
    onKeyDown(e) {
        if (e.key === 'Escape' && this.state !== ButtonState.Released) {
            this.cancel();
            this.reset();
        }
    }
    press(e) {
        this.preventDefault = false;
        this.stopPropagation = false;
        const targetPath = e.composedPath();
        const underPointer = document.elementsFromPoint(e.clientX, e.clientY);
        const { data, window } = (0, DataForSensor_1.findTargetElementData)(targetPath, underPointer, DataForSensor_1.SymDataForSensor, ['button']);
        const originData = data === null || data === void 0 ? void 0 : data.button;
        if (originData) {
            this.state = ButtonState.Pressed;
            this.pointerButton = (0, BasePointerSensor_1.extractPointerButton)(e);
            this.originData = originData;
            this.modifiers = (0, KeyboardSensor_1.extractModifierKeys)(e);
            this.positionX = e.clientX;
            this.positionY = e.clientY;
            this.revision++;
        }
        (0, reactronic_1.standalone)(() => {
            var _a;
            (_a = this.windowSensor) === null || _a === void 0 ? void 0 : _a.setActiveWindow(window, 'button');
        });
    }
    selecting(e) {
        this.updateSensorData(e);
        this.state = ButtonState.Selecting;
        this.selected = false;
    }
    select(e) {
        this.updateSensorData(e);
        this.state = ButtonState.Selected;
        this.selectedX = e.clientX;
        this.selectedY = e.clientY;
        this.selected = true;
    }
    release() {
        this.preventDefault = false;
        this.stopPropagation = false;
        this.state = ButtonState.Released;
        this.revision++;
    }
    cancel() {
        this.state = ButtonState.Released;
        this.selected = false;
        this.revision++;
    }
    reset() {
        this.preventDefault = false;
        this.stopPropagation = false;
        this.state = ButtonState.Released;
        this.originData = undefined;
        this.selectedData = undefined;
        this.pointerButton = BasePointerSensor_1.PointerButton.None;
        this.positionX = Infinity;
        this.positionY = Infinity;
        this.selectedX = Infinity;
        this.selectedY = Infinity;
        this.modifiers = KeyboardSensor_1.KeyboardModifiers.None;
        this.selected = false;
    }
    updateSensorData(e) {
        this.preventDefault = false;
        this.stopPropagation = false;
        this.modifiers = (0, KeyboardSensor_1.extractModifierKeys)(e);
        this.positionX = e.clientX;
        this.positionY = e.clientY;
        this.revision++;
    }
}
__decorate([
    reactronic_1.transaction,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Boolean]),
    __metadata("design:returntype", void 0)
], ButtonSensor.prototype, "listen", null);
__decorate([
    reactronic_1.transaction,
    (0, reactronic_1.options)({ trace: reactronic_1.TraceLevel.Silent }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [PointerEvent]),
    __metadata("design:returntype", void 0)
], ButtonSensor.prototype, "press", null);
__decorate([
    reactronic_1.transaction,
    (0, reactronic_1.options)({ reentrance: reactronic_1.Reentrance.CancelPrevious, trace: reactronic_1.TraceLevel.Silent }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [PointerEvent]),
    __metadata("design:returntype", void 0)
], ButtonSensor.prototype, "selecting", null);
__decorate([
    reactronic_1.transaction,
    (0, reactronic_1.options)({ trace: reactronic_1.TraceLevel.Silent }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [PointerEvent]),
    __metadata("design:returntype", void 0)
], ButtonSensor.prototype, "select", null);
__decorate([
    reactronic_1.transaction,
    (0, reactronic_1.options)({ trace: reactronic_1.TraceLevel.Silent }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ButtonSensor.prototype, "release", null);
__decorate([
    reactronic_1.transaction,
    (0, reactronic_1.options)({ trace: reactronic_1.TraceLevel.Silent }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ButtonSensor.prototype, "cancel", null);
__decorate([
    reactronic_1.transaction,
    (0, reactronic_1.options)({ trace: reactronic_1.TraceLevel.Silent }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ButtonSensor.prototype, "reset", null);
exports.ButtonSensor = ButtonSensor;
